package de.ozgcloud.bescheid;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.Collections;
import java.util.List;
import java.util.Optional;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.bescheid.attacheditem.AttachedItem;
import de.ozgcloud.bescheid.attacheditem.AttachedItemMapper;
import de.ozgcloud.bescheid.attacheditem.AttachedItemService;
import de.ozgcloud.bescheid.attacheditem.AttachedItemTestFactory;
import de.ozgcloud.bescheid.vorgang.VorgangId;
import de.ozgcloud.command.CommandTestFactory;
import io.grpc.stub.StreamObserver;

class BescheidGrpcServiceTest {

	@Spy
	@InjectMocks
	private BescheidGrpcService service;

	@Mock
	private AttachedItemService attachedItemService;
	@Mock
	private AttachedItemMapper attachedItemMapper;
	@Mock
	private BescheidService bescheidService;
	@Mock
	private GrpcBescheidMapper grpcBescheidMapper;

	@Nested
	class TestGetBescheidDraft {

		@Mock
		private GrpcGetBescheidDraftRequest request;
		@Mock
		private StreamObserver<GrpcGetBescheidDraftResponse> responseObserver;
		@Captor
		private ArgumentCaptor<GrpcGetBescheidDraftResponse> responseCaptor;

		private static final GrpcGetBescheidDraftResponse response = GrpcGetBescheidDraftResponse.newBuilder().build();

		@Test
		void shouldBuildResponse() {
			when(attachedItemMapper.toBescheid(any())).thenReturn(GrpcBescheidTestFactory.create());
			var bescheidItem = AttachedItemTestFactory.createBescheid();
			when(attachedItemService.findBescheidItem(any())).thenReturn(Optional.of(bescheidItem));

			service.getBescheidDraft(request, responseObserver);

			verify(service).buildGetBescheidDraftResponse(bescheidItem);
		}

		@Test
		void shouldCallNext() {
			when(attachedItemMapper.toBescheid(any())).thenReturn(GrpcBescheidTestFactory.create());
			when(service.buildGetBescheidDraftResponse(any())).thenReturn(response);
			when(attachedItemService.findBescheidItem(any())).thenReturn(Optional.of(AttachedItemTestFactory.createBescheid()));

			service.getBescheidDraft(request, responseObserver);

			verify(responseObserver).onNext(response);
		}

		@Test
		void shouldCallNextWhenNoBescheid() {
			service.getBescheidDraft(request, responseObserver);

			verify(responseObserver).onNext(responseCaptor.capture());
			assertThat(responseCaptor.getValue().hasBescheid()).isFalse();
		}

		@Test
		void shouldCallCompleted() {
			service.getBescheidDraft(request, responseObserver);

			verify(responseObserver).onCompleted();
		}

	}

	@Nested
	class TestBuildGetBescheidDraftResponse {

		@Test
		void shouldCallMapper() {
			when(attachedItemMapper.toBescheid(any())).thenReturn(GrpcBescheidTestFactory.create());
			var bescheid = AttachedItemTestFactory.createBescheid();

			service.buildGetBescheidDraftResponse(bescheid);

			verify(attachedItemMapper).toBescheid(bescheid);
		}

		@Test
		void shouldSetBescheid() {
			var grpcBescheid = GrpcBescheidTestFactory.create();
			when(attachedItemMapper.toBescheid(any())).thenReturn(grpcBescheid);

			var response = service.buildGetBescheidDraftResponse(AttachedItemTestFactory.createBescheid());

			assertThat(response.getBescheid()).isEqualTo(grpcBescheid);
		}
	}

	@Nested
	class TestGetConfig {

		private static final GrpcBescheidManagerConfigResponse RESPONSE = GrpcBescheidManagerConfigResponse.newBuilder().build();

		@Mock
		private GrpcBescheidManagerConfigRequest request;
		@Mock
		private StreamObserver<GrpcBescheidManagerConfigResponse> responseObserver;
		@Captor
		private ArgumentCaptor<GrpcBescheidManagerConfigResponse> responseCaptor;

		@Test
		void shouldCallBescheidService() {
			getConfig();

			verify(bescheidService).getConfig();
		}

		@Test
		void shouldCallGrpcBescheidMapper() {
			var config = BescheidManagerConfigTestFactory.create();
			when(bescheidService.getConfig()).thenReturn(config);

			getConfig();

			verify(grpcBescheidMapper).fromBescheidManagerConfig(config);
		}

		@Test
		void shouldCallNext() {
			when(grpcBescheidMapper.fromBescheidManagerConfig(any())).thenReturn(RESPONSE);

			getConfig();

			verify(responseObserver).onNext(RESPONSE);
		}

		@Test
		void shouldCallCompleted() {
			getConfig();

			verify(responseObserver).onCompleted();
		}

		private void getConfig() {
			service.getConfig(request, responseObserver);
		}
	}

	@Nested
	class TestGetAll {

		private static final GrpcGetAllBescheidResponse RESPONSE = GrpcGetAllBescheidResponse.newBuilder().build();
		@Mock
		private GrpcGetAllBescheidRequest request;
		@Mock
		private StreamObserver<GrpcGetAllBescheidResponse> responseObserver;
		@Captor
		private ArgumentCaptor<GrpcGetAllBescheidResponse> responseCaptor;
		@Captor
		private ArgumentCaptor<VorgangId> vorgangIdCaptor;

		@BeforeEach
		void init() {
			when(request.getVorgangId()).thenReturn(CommandTestFactory.VORGANG_ID);
			doReturn(RESPONSE).when(service).buildGetAllBescheidResponse(anyCollection());
		}

		@Test
		void shouldCallFindAllBescheid() {
			service.getAll(request, responseObserver);

			verify(attachedItemService).findAllBescheid(vorgangIdCaptor.capture());
			assertThat(vorgangIdCaptor.getValue()).hasToString(CommandTestFactory.VORGANG_ID);
		}

		@Test
		void shouldCallBuildResponse() {
			var expectedBescheidItem = AttachedItemTestFactory.createBescheid();
			when(attachedItemService.findAllBescheid(any())).thenReturn(List.of(expectedBescheidItem));

			service.getAll(request, responseObserver);

			verify(service).buildGetAllBescheidResponse(List.of(expectedBescheidItem));
		}

		@Test
		void shouldCallOnNext() {
			service.getAll(request, responseObserver);

			verify(responseObserver).onNext(RESPONSE);
		}

		@Test
		void shouldCallOnCompleted() {
			service.getAll(request, responseObserver);

			verify(responseObserver).onCompleted();
		}
	}

	@Nested
	class TestBuildResponseAllBescheid {

		private static final GrpcBescheid GRPC_BESCHEID = GrpcBescheidTestFactory.create();

		@Mock
		private AttachedItem attachedItem;

		@BeforeEach
		void init() {
			doReturn(Collections.singletonList(GRPC_BESCHEID)).when(service).mapBescheid(anyCollection());
		}

		@Test
		void shouldCallMapBescheid() {
			var bescheidItems = Collections.singleton(attachedItem);

			service.buildGetAllBescheidResponse(bescheidItems);

			verify(service).mapBescheid(bescheidItems);
		}

		@Test
		void shouldReturnResponse() {
			var bescheidItems = Collections.singleton(attachedItem);

			var response = service.buildGetAllBescheidResponse(bescheidItems);

			assertThat(response.getBescheidList()).containsExactly(GRPC_BESCHEID);
		}
	}

	@Nested
	class TestMapBescheid {

		private static final GrpcBescheid GRPC_BESCHEID = GrpcBescheidTestFactory.create();

		@Mock
		private AttachedItem attachedItem;

		@BeforeEach
		void init() {
			when(attachedItemMapper.toBescheid(attachedItem)).thenReturn(GRPC_BESCHEID);
		}

		@Test
		void shouldCallMapper() {
			service.mapBescheid(Collections.singleton(attachedItem));

			verify(attachedItemMapper).toBescheid(attachedItem);
		}

		@Test
		void shouldSetBescheid() {
			var result = service.mapBescheid(Collections.singleton(attachedItem));

			assertThat(result).containsExactly(GRPC_BESCHEID);
		}

	}
}
