/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.clientattribute;

import static de.ozgcloud.vorgang.clientattribute.ClientAttributeTestFactory.*;
import static org.assertj.core.api.Assertions.*;

import java.util.Optional;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mapstruct.factory.Mappers;
import org.mockito.InjectMocks;
import org.mockito.Mock;

import de.ozgcloud.vorgang.callcontext.CallContextTestFactory;
import de.ozgcloud.vorgang.callcontext.CurrentUserService;
import de.ozgcloud.vorgang.grpc.clientAttribute.GrpcClientAttributeValue;

class ClientAttributeMapperTest {

	@InjectMocks
	private ClientAttributeMapper mapper = Mappers.getMapper(ClientAttributeMapper.class);
	@Mock
	private CurrentUserService userService;

	@Nested
	class TestMapFromGrpc {

		@Test
		void shouldSetClientName() {
			var grpcAttribute = GrpcClientAttributeTestFactory.createBuilder().build();

			var attribute = mapper.mapFromGrpc(grpcAttribute);

			assertThat(attribute.getClientName()).isEqualTo(CallContextTestFactory.CLIENT);
		}

		@Nested
		class ShouldMapValue {

			@Test
			void withIntValue() {
				var grpcAttribute = GrpcClientAttributeTestFactory.withValue(GrpcClientAttributeValue.newBuilder().setIntValue(INT_VALUE).build());

				var attribute = mapper.mapFromGrpc(grpcAttribute);

				assertThat(attribute).usingRecursiveComparison()
						.isEqualTo(ClientAttributeTestFactory.createBuilder().intValue(Optional.of(INT_VALUE)).build());
			}

			@Test
			void withStringValue() {
				var grpcAttribute = GrpcClientAttributeTestFactory
						.withValue(GrpcClientAttributeValue.newBuilder().setStringValue(STR_VALUE).build());

				var attribute = mapper.mapFromGrpc(grpcAttribute);

				assertThat(attribute).usingRecursiveComparison()
						.isEqualTo(ClientAttributeTestFactory.createBuilder().intValue(Optional.empty()).stringValue(Optional.of(STR_VALUE)).build());
			}

			@Test
			void withBooleanValue() {
				var grpcAttribute = GrpcClientAttributeTestFactory.withValue(GrpcClientAttributeValue.newBuilder().setBoolValue(true).build());

				var attribute = mapper.mapFromGrpc(grpcAttribute);

				assertThat(attribute).usingRecursiveComparison()
						.isEqualTo(ClientAttributeTestFactory.createBuilder().intValue(Optional.empty()).boolValue(Optional.of(BOOL_VALUE)).build());
			}

			@Test
			void withDoubleValue() {
				var grpcAttribute = GrpcClientAttributeTestFactory
						.withValue(GrpcClientAttributeValue.newBuilder().setDoubleValue(DOUBLE_VALUE).build());

				var attribute = mapper.mapFromGrpc(grpcAttribute);

				assertThat(attribute).usingRecursiveComparison()
						.isEqualTo(
								ClientAttributeTestFactory.createBuilder().intValue(Optional.empty()).doubleValue(Optional.of(DOUBLE_VALUE)).build());
			}

		}

		@Test
		void shouldHaveEmptyValuesIfMissingValue() {
			var grpcAttribute = GrpcClientAttributeTestFactory.withValue(GrpcClientAttributeValue.newBuilder().clearValue().build());

			var attribute = mapper.mapFromGrpc(grpcAttribute);

			assertThat(attribute.getBoolValue()).isEmpty();
			assertThat(attribute.getIntValue()).isEmpty();
			assertThat(attribute.getDoubleValue()).isEmpty();
			assertThat(attribute.getStringValue()).isEmpty();
		}

		@Test
		void shouldHaveEmptyValueIfNoValueSet() {
			var grpcAttribute = GrpcClientAttributeTestFactory.withValue(GrpcClientAttributeValue.newBuilder()
					.clearIntValue().clearBoolValue().clearStringValue().clearDoubleValue().build());

			var attribute = mapper.mapFromGrpc(grpcAttribute);

			assertThat(attribute.getBoolValue()).isEmpty();
			assertThat(attribute.getIntValue()).isEmpty();
			assertThat(attribute.getDoubleValue()).isEmpty();
			assertThat(attribute.getStringValue()).isEmpty();
		}
	}

	@Nested
	class TestMapClientAttributes {

		@Test
		void shouldMapEmptyMap() {
			var result = mapper.mapClientAttributes(new ClientAttributesMap());

			assertThat(result).isEmpty();
		}

		@Test
		void shouldBeEmptyIfNoAttributes() {
			var attributes = new ClientAttributesMap();
			attributes.put(CallContextTestFactory.CLIENT, new ClientAttributeMap());

			var result = mapper.mapClientAttributes(attributes);

			assertThat(result).isEmpty();
		}

		@Test
		void shouldHaveAttribute() {
			var result = mapper.mapClientAttributes(ClientAttributesMapTestFactory.create());

			assertThat(result).hasSize(1).contains(GrpcClientAttributeTestFactory.create());
		}

		@Test
		void shouldHaveStringValue() {
			var result = mapper.mapClientAttributes(ClientAttributesMapTestFactory.createWithAttribute(
					ClientAttributeTestFactory.createBuilder().intValue(Optional.empty()).stringValue(Optional.of(STR_VALUE)).build()));

			assertThat(result).hasSize(1);
			assertThat(result.get(0).getValue().getStringValue()).isEqualTo(STR_VALUE);
		}

		@Test
		void shouldHaveDoubleValue() {
			var result = mapper.mapClientAttributes(ClientAttributesMapTestFactory.createWithAttribute(
					ClientAttributeTestFactory.createBuilder().intValue(Optional.empty()).doubleValue(Optional.of(DOUBLE_VALUE)).build()));

			assertThat(result).hasSize(1);
			assertThat(result.get(0).getValue().getDoubleValue()).isEqualTo(DOUBLE_VALUE);
		}

		@Test
		void shouldHaveBoolValue() {
			var result = mapper.mapClientAttributes(ClientAttributesMapTestFactory.createWithAttribute(
					ClientAttributeTestFactory.createBuilder().intValue(Optional.empty()).boolValue(Optional.of(BOOL_VALUE)).build()));

			assertThat(result).hasSize(1);
			assertThat(result.get(0).getValue().getBoolValue()).isEqualTo(BOOL_VALUE);
		}

		@Test
		void shouldByFineWithNoValue() {
			var result = mapper.mapClientAttributes(ClientAttributesMapTestFactory.createWithAttribute(
					ClientAttributeTestFactory.createBuilder().intValue(Optional.empty()).build()));

			assertThat(result).hasSize(1);
			assertThat(result.get(0).getValue()).isNotNull();
		}

	}
}
