/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.collaboration;

import static org.assertj.core.api.Assertions.*;
import static org.awaitility.Awaitility.*;
import static org.mockito.Mockito.*;

import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.Mock;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.data.mongodb.core.MongoOperations;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.test.annotation.DirtiesContext;

import de.ozgcloud.apilib.user.OzgCloudUserId;
import de.ozgcloud.apilib.user.OzgCloudUserProfile;
import de.ozgcloud.apilib.user.OzgCloudUserProfileService;
import de.ozgcloud.command.Command;
import de.ozgcloud.command.CommandStatus;
import de.ozgcloud.common.test.DataITCase;
import de.ozgcloud.vorgang.attached_item.VorgangAttachedItem;
import de.ozgcloud.vorgang.callcontext.CallContext;
import de.ozgcloud.vorgang.callcontext.WithMockCustomUser;
import de.ozgcloud.vorgang.command.CommandService;
import de.ozgcloud.vorgang.command.CommandTestFactory;
import de.ozgcloud.vorgang.command.CreateCommandRequest;
import de.ozgcloud.vorgang.vorgang.Vorgang;
import de.ozgcloud.vorgang.vorgang.VorgangHead;
import de.ozgcloud.vorgang.vorgang.VorgangTestFactory;
import de.ozgcloud.vorgang.vorgang.ZustaendigeStelleTestFactory;

@SpringBootTest(properties = {
		"grpc.server.inProcessName=test",
		"grpc.client.vorgang-manager.address=in-process:test",
		"grpc.client.ozgcloud-command-manager.address=in-process:test",
})
@DataITCase
@WithMockCustomUser
@DirtiesContext
class CollaborationITCase {

	@Autowired
	private CommandService commandService;

	@Autowired
	private MongoOperations mongoOperations;

	@MockBean
	private OzgCloudUserProfileService ozgCloudUserProfileService;
	@Mock
	private OzgCloudUserProfile ozgCloudUserProfile;

	@BeforeEach
	void init() {
		mongoOperations.dropCollection(Command.class);
		mongoOperations.dropCollection(Vorgang.class);
		mongoOperations.dropCollection(VorgangAttachedItem.class);
		when(ozgCloudUserProfile.getId()).thenReturn(OzgCloudUserId.from(CommandTestFactory.CREATED_BY));
		when(ozgCloudUserProfileService.getById(any())).thenReturn(ozgCloudUserProfile);
	}

	@Nested
	class TestCreateCollaborationVorgang {

		private static final String FIELD_COLLABORATION_VORGANG_ID = "collaborationVorgangId";
		private static final String TITEL = "Collaboration Vorgang";
		private static final String ANFRAGE = "Anfrage";

		private String vorgangId;

		@BeforeEach
		void init() {
			vorgangId = mongoOperations.save(VorgangTestFactory.createBuilder().id(null).version(0L).build()).getId();
		}

		@Test
		void shouldSetCollaborationVorgangId() {
			var command = commandService.createCommand(buildCreateCollaborationVorgangCommand(vorgangId));

			waitUntilCommandFinished(command.getId());

			var collaborationRequests = loadCollaborationRequest(vorgangId);
			assertThat(collaborationRequests).hasSize(1).first().extracting(VorgangAttachedItem::getItem, MAP)
					.containsKey(FIELD_COLLABORATION_VORGANG_ID);
		}

		@Test
		void shouldCreateCollaborationVorgang() {
			var command = commandService.createCommand(buildCreateCollaborationVorgangCommand(vorgangId));

			waitUntilCommandFinished(command.getId());

			var collaborationVorgang = loadCollaborationVorgang(vorgangId);
			assertThat(collaborationVorgang.getHeader()).extracting(VorgangHead::getCollaborationLevel)
					.isEqualTo(CreateCollaborationVorgangRequestTestFactory.COLLABORATION_LEVEL);
			assertThat(collaborationVorgang.getHeader()).extracting(VorgangHead::getOrganisationsEinheitId)
					.isEqualTo(ZustaendigeStelleTestFactory.ORGANISATIONSEINHEIT_ID);
			assertThat(collaborationVorgang.getClientAttributes()).isEmpty();
		}

		private CreateCommandRequest buildCreateCollaborationVorgangCommand(String vorgangId) {
			return CreateCommandRequest.builder()
					.callContext(CallContext.builder().client("test").build())
					.vorgangId(vorgangId)
					.relationId(vorgangId)
					.order("CREATE_COLLABORATION_REQUEST")
					.bodyObject(Map.of(
							"titel", TITEL,
							"anfrage", ANFRAGE,
							"zustaendigeStelle", ZustaendigeStelleTestFactory.ORGANISATIONSEINHEIT_ID
					))
					.build();
		}

		private void waitUntilCommandFinished(String commandId) {
			await().atMost(60, TimeUnit.SECONDS).until(
					() -> mongoOperations.findById(commandId, Command.class),
					command -> command.getStatus() == CommandStatus.FINISHED
			);
		}

		private Vorgang loadCollaborationVorgang(String sourceVorgangId) {
			var collaborationVorgangId = loadCollaborationRequest(sourceVorgangId).getFirst().getItem().get(FIELD_COLLABORATION_VORGANG_ID);
			return mongoOperations.findById(collaborationVorgangId, Vorgang.class);
		}

		private List<VorgangAttachedItem> loadCollaborationRequest(String vorgangId) {
			var query = new Query(new Criteria().andOperator(
					Criteria.where(VorgangAttachedItem.FIELDNAME_VORGANG_ID).is(vorgangId),
					Criteria.where(VorgangAttachedItem.FIELDNAME_ITEM_NAME).is("CollaborationRequest")
			));
			return mongoOperations.find(query, VorgangAttachedItem.class);
		}
	}

}