/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.collaboration;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.time.ZonedDateTime;
import java.time.temporal.ChronoUnit;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.springframework.context.ApplicationEventPublisher;

import de.ozgcloud.command.VorgangCreatedEvent;
import de.ozgcloud.vorgang.servicekonto.ServiceKonto;
import de.ozgcloud.vorgang.servicekonto.ServiceKontoTestFactory;
import de.ozgcloud.vorgang.vorgang.Vorgang;
import de.ozgcloud.vorgang.vorgang.VorgangHead;
import de.ozgcloud.vorgang.vorgang.VorgangHeadTestFactory;
import de.ozgcloud.vorgang.vorgang.VorgangTestFactory;
import de.ozgcloud.vorgang.vorgang.ZustaendigeStelleTestFactory;

class CollaborationServiceTest {

	@Spy
	@InjectMocks
	private CollaborationService service;

	@Mock
	private CollaborationRepository collaborationRepository;
	@Mock
	private ApplicationEventPublisher publisher;

	@Nested
	class TestCreateCollaborationVorgang {

		private static final CreateCollaborationVorgangRequest CREATE_COLLABORATION_VORGANG_REQUEST = CreateCollaborationVorgangRequestTestFactory.create();
		private static final String COLLABORATION_VORGANG_ID = "collaboration-vorgang-id";
		private static final Vorgang SAVED_VORGANG = VorgangTestFactory.createBuilder().id(COLLABORATION_VORGANG_ID).build();

		@Mock
		private Vorgang collaborationVorgang;
		@Captor
		private ArgumentCaptor<VorgangCreatedEvent> eventCaptor;

		@BeforeEach
		void init() {
			doReturn(collaborationVorgang).when(service).buildCollaborationVorgang(any());
			when(collaborationRepository.save(any())).thenReturn(SAVED_VORGANG);
		}

		@Test
		void shouldCallBuildCollaborationVorgang() {
			createCollaborationVorgang();

			verify(service).buildCollaborationVorgang(CREATE_COLLABORATION_VORGANG_REQUEST);
		}

		@Test
		void shouldCallSaveVorgang() {
			createCollaborationVorgang();

			verify(collaborationRepository).save(collaborationVorgang);
		}

		@Test
		void shouldPublishEvent() {
			createCollaborationVorgang();

			verifyNoInteractions(publisher);
			// TODO: aktiviere, wenn VorgangCreatedEvent veröffentlicht wird
			// 	verify(publisher).publishEvent(eventCaptor.capture());
			// 	assertThat(eventCaptor.getValue().getSource()).isEqualTo(COLLABORATION_VORGANG_ID);
		}

		@Test
		void shouldReturnVorgang() {
			var result = createCollaborationVorgang();

			assertThat(result).isSameAs(SAVED_VORGANG);
		}

		private Vorgang createCollaborationVorgang() {
			return service.createCollaborationVorgang(CREATE_COLLABORATION_VORGANG_REQUEST);
		}
	}

	@Nested
	class TestBuildCollaborationVorgang {

		private static final CreateCollaborationVorgangRequest COLLABORATION_REQUEST = CreateCollaborationVorgangRequestTestFactory.createBuilder()
				.vorgang(VorgangTestFactory.createBuilder().formEngineName(VorgangTestFactory.FORM_ENGINE_NAME).build()).build();

		private static final VorgangHead UPDATED_HEAD = VorgangHeadTestFactory.create();

		@BeforeEach
		void init() {
			doReturn(UPDATED_HEAD).when(service).getHeader(any());
		}

		@Test
		void shouldSetVorgangName() {
			var result = buildCollaborationVorgang();

			assertThat(result.getName()).isEqualTo(VorgangTestFactory.NAME);
		}

		@Test
		void shouldSetVorgangNummer() {
			var result = buildCollaborationVorgang();

			assertThat(result.getNummer()).isEqualTo(VorgangTestFactory.VORGANG_NUMMER);
		}

		@Test
		void shouldSetFormEngineName() {
			var result = buildCollaborationVorgang();

			assertThat(result.getFormEngineName()).isEqualTo(VorgangTestFactory.FORM_ENGINE_NAME);
		}

		@Test
		void shouldSetAktenzeichen() {
			var result = buildCollaborationVorgang();

			assertThat(result.getAktenzeichen()).isEqualTo(VorgangTestFactory.AKTENZEICHEN);
		}

		@Test
		void shouldCallGetHeader() {
			buildCollaborationVorgang();

			verify(service).getHeader(COLLABORATION_REQUEST);
		}

		@Test
		void shouldSetHeader() {
			var result = buildCollaborationVorgang();

			assertThat(result.getHeader()).isEqualTo(UPDATED_HEAD);
		}

		@Test
		void shouldSetEingangs() {
			var result = buildCollaborationVorgang();

			assertThat(result.getEingangs()).containsExactly(VorgangTestFactory.EINGANG);
		}

		@Test
		void shouldSetCreatedAt() {
			var result = buildCollaborationVorgang();

			assertThat(result.getCreatedAt()).isCloseTo(ZonedDateTime.now(), within(1, ChronoUnit.SECONDS));
		}

		@Test
		void shouldSetInCreationFalse() {
			var result = buildCollaborationVorgang();

			assertThat(result.isInCreation()).isFalse();
		}

		@Test
		void shouldSetStatusNeu() {
			var result = buildCollaborationVorgang();

			assertThat(result.getStatus()).isEqualTo(Vorgang.Status.NEU);
		}

		@Test
		void shouldNotSetAssignedTo() {
			var result = buildCollaborationVorgang();

			assertThat(result.getAssignedTo()).isNull();
		}

		@Test
		void shouldNotSetClientAttributes() {
			var result = buildCollaborationVorgang();

			assertThat(result.getClientAttributes()).isEmpty();
		}

		@Test
		void shouldNotSetForwardings() {
			var result = buildCollaborationVorgang();

			assertThat(result.getForwardings()).isEmpty();
		}

		private Vorgang buildCollaborationVorgang() {
			return service.buildCollaborationVorgang(COLLABORATION_REQUEST);
		}
	}

	@Nested
	class TestGetHeader {

		private static final ServiceKonto SERVICE_KONTO = ServiceKontoTestFactory.create();

		@Test
		void shouldSetZustaendigeStelle() {
			var result = getHeader();

			assertThat(result.getOrganisationsEinheitId()).isEqualTo(ZustaendigeStelleTestFactory.ORGANISATIONSEINHEIT_ID);
		}

		@Test
		void shouldSetCollaborationLevel() {
			var result = getHeader();

			assertThat(result.getCollaborationLevel()).isEqualTo(CreateCollaborationVorgangRequestTestFactory.COLLABORATION_LEVEL);
		}

		@Test
		void shouldKeepServiceKonto() {
			var result = getHeader();

			assertThat(result.getServiceKonto()).isSameAs(SERVICE_KONTO);
		}

		private VorgangHead getHeader() {
			return service.getHeader(CreateCollaborationVorgangRequestTestFactory.createBuilder().vorgang(VorgangTestFactory.createBuilder().header(
					VorgangHeadTestFactory.createBuilder().serviceKonto(SERVICE_KONTO).build()).build()).build());
		}
	}

}